package com.huiju.estateagents.eContract.service.impl;

import com.fadada.sdk.client.FddClientBase;
import com.fadada.sdk.util.crypt.FddEncryptTool;
import com.fadada.sdk.util.http.HttpsUtil;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.config.FadadaProperties;
import com.huiju.estateagents.entity.TaMiniapp;
import com.huiju.estateagents.mapper.TaMiniappMapper;
import org.apache.http.message.BasicNameValuePair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

@Component
public class TaCompanySealUtil {

    @Autowired
    private FadadaProperties fadadaProperties;

    @Autowired
    private TaMiniappMapper taMiniappMapper;

    public String addSignaTure(String customerId, String imgUrl, String miniappId) throws Exception {
        FddClientBase base = new FddClientBase(fadadaProperties.getAppId(), fadadaProperties.getAppSecret(), fadadaProperties.getVersion(), getFddServerHost(miniappId,fadadaProperties.getServeHost()));
        String base64Url = FddEncryptTool.ImageToBase64ByOnline(imgUrl);
        return base.invokeaddSignature(customerId, base64Url);
    }

    public String replaceSignature(String customerId, String signatureId, String imgUrl, String miniappId){
        String base64Url = FddEncryptTool.ImageToBase64ByOnline(imgUrl);
        ArrayList params = new ArrayList();

        try {
            String timeStamp = HttpsUtil.getTimeStamp();
            String sha1 = FddEncryptTool.sha1(fadadaProperties.getAppId() + FddEncryptTool.md5Digest(timeStamp) + FddEncryptTool.sha1(fadadaProperties.getAppSecret() + customerId + signatureId + base64Url));
            String msgDigest = new String(FddEncryptTool.Base64Encode(sha1.getBytes()));
            params.add(new BasicNameValuePair("customer_id", customerId));
            params.add(new BasicNameValuePair("signature_img_base64", base64Url));
            params.add(new BasicNameValuePair("app_id", fadadaProperties.getAppId()));
            params.add(new BasicNameValuePair("signature_id", signatureId));
            params.add(new BasicNameValuePair("timestamp", timeStamp));
            params.add(new BasicNameValuePair("v", fadadaProperties.getVersion()));
            params.add(new BasicNameValuePair("msg_digest", msgDigest));
        } catch (Exception var7) {
            var7.printStackTrace();
            throw new RuntimeException(var7);
        }

        return HttpsUtil.doPost(getFddServerHost(miniappId,fadadaProperties.getServeHost()) + "replace_signature.api", params);
    }

    public String delSignature(String customerId, String signatureId, String miniappId){
        ArrayList params = new ArrayList();

        try {
            String timeStamp = HttpsUtil.getTimeStamp();
            String sha1 = FddEncryptTool.sha1(fadadaProperties.getAppId() + FddEncryptTool.md5Digest(timeStamp) + FddEncryptTool.sha1(fadadaProperties.getAppSecret() + customerId + signatureId));
            String msgDigest = new String(FddEncryptTool.Base64Encode(sha1.getBytes()));
            params.add(new BasicNameValuePair("customer_id", customerId));
            params.add(new BasicNameValuePair("app_id", fadadaProperties.getAppId()));
            params.add(new BasicNameValuePair("timestamp", timeStamp));
            params.add(new BasicNameValuePair("v", fadadaProperties.getVersion()));
            params.add(new BasicNameValuePair("signature_id", signatureId));
            params.add(new BasicNameValuePair("msg_digest", msgDigest));
        } catch (Exception var7) {
            var7.printStackTrace();
            throw new RuntimeException(var7);
        }

        return HttpsUtil.doPost(getFddServerHost(miniappId,fadadaProperties.getServeHost()) + "remove_signature.api", params);
    }

    public String autoAuthCompanySeal(String transactionId, Integer authType, String contractId, String customerId, String companyId, String miniappId) throws Exception {
        String timeStamp = HttpsUtil.getTimeStamp();
        String sha1 = FddEncryptTool.sha1(fadadaProperties.getAppId() + FddEncryptTool.md5Digest(transactionId + timeStamp) + FddEncryptTool.sha1(fadadaProperties.getAppSecret() + customerId));
        String msgDigest = new String(FddEncryptTool.Base64Encode(sha1.getBytes()));
        StringBuilder stringBuilder = new StringBuilder(getFddServerHost(miniappId,fadadaProperties.getServeHost()) + "before_authsign.api?");
        stringBuilder.append("app_id="+fadadaProperties.getAppId()+"&");
        stringBuilder.append("customer_id="+customerId+"&");
        stringBuilder.append("timestamp="+timeStamp+"&");
        stringBuilder.append("v="+fadadaProperties.getVersion()+"&");
        stringBuilder.append("transaction_id="+transactionId+"&");
        stringBuilder.append("auth_type="+authType+"&");
        stringBuilder.append("contract_id="+contractId+"&");
        stringBuilder.append("msg_digest="+msgDigest+"&");
        stringBuilder.append("notify_url="+fadadaProperties.getAuthNotifyUrl()+"&");
        stringBuilder.append("return_url="+fadadaProperties.getAuthReturnUrl() + companyId);

        return stringBuilder.toString();
    }

    public String getAuthStatus(String customerId, String miniappId){
        ArrayList params = new ArrayList();

        try {
            String timeStamp = HttpsUtil.getTimeStamp();
            String sha1 = FddEncryptTool.sha1(fadadaProperties.getAppId() + FddEncryptTool.md5Digest(timeStamp) + FddEncryptTool.sha1(fadadaProperties.getAppSecret() + customerId));
            String msgDigest = new String(FddEncryptTool.Base64Encode(sha1.getBytes()));
            params.add(new BasicNameValuePair("customer_id", customerId));
            params.add(new BasicNameValuePair("app_id", fadadaProperties.getAppId()));
            params.add(new BasicNameValuePair("timestamp", timeStamp));
            params.add(new BasicNameValuePair("v", fadadaProperties.getVersion()));
            params.add(new BasicNameValuePair("msg_digest", msgDigest));
        } catch (Exception var7) {
            var7.printStackTrace();
            throw new RuntimeException(var7);
        }

        return HttpsUtil.doPost(getFddServerHost(miniappId,fadadaProperties.getServeHost()) + "get_auth_status.api", params);
    }

    /**
     * 替换法大大服务器地址
     * @return
     */
    private String getFddServerHost(String miniappId,String subUrl) {
        TaMiniapp taMiniapp = taMiniappMapper.selectById(miniappId);
        int index = subUrl.indexOf(File.separator);
        int twoIndex = subUrl.indexOf(File.separator,index);
        int threeIndex = subUrl.indexOf(File.separator,twoIndex);
        String newUrl = StringUtils.isEmpty(taMiniapp.getFddServerHost()) ? subUrl : taMiniapp.getFddServerHost() + subUrl.substring(threeIndex);
        return newUrl;
    }
}
